@@ -5,6 +5,7 @@ from django.utils import timezone
 from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.fields import GenericRelation
 from django.db import connection
+from django.dispatch import receiver
 
 from django.contrib.gis.db.models.functions import Centroid
 
@@ -22,7 +23,7 @@ from djangorestframework_camel_case.util import underscoreize
 
 from PIL import Image, ImageOps
 
-import uuid, json, os
+import uuid, json, os, shutil
 
 from .json_schemas import OBSERVATION_FORM_SCHEMA
 
@@ -341,8 +342,8 @@ class Dataset(ModelWithTaxon):
             
         return queryset
 
-
-    def thumbnail_url(self):
+    @property
+    def thumbnail(self):
         image = DatasetImages.objects.filter(dataset=self).first()
 
         if image:
@@ -378,6 +379,11 @@ class Dataset(ModelWithTaxon):
 
         if created == True:
             self.validate()
+            
+            
+    def get_app(self):
+        app = App.objects.filter(uuid=self.app_uuid).first()
+        return app
      
 
     def __str__(self):
@@ -388,6 +394,7 @@ class Dataset(ModelWithTaxon):
 
     class Meta:
         ordering = ['-pk']
+        verbose_name = _('Dataset')
 
 
 
@@ -422,6 +429,7 @@ class DatasetValidationRoutine(ModelWithTaxon):
     class Meta:
         unique_together = ('app', 'validation_class')
         ordering = ['position']
+        verbose_name = _('Dataset Validation Routine')
 
 
 
@@ -456,7 +464,7 @@ class DatasetImages(models.Model):
 
     resized_folder_name = 'resized'
 
-    dataset = models.ForeignKey(Dataset, on_delete=models.CASCADE)
+    dataset = models.ForeignKey(Dataset, on_delete=models.CASCADE, related_name='images')
     field_uuid = models.UUIDField()
     image = models.ImageField(max_length=255, upload_to=dataset_image_path)
 
@@ -522,17 +530,24 @@ class DatasetImages(models.Model):
         return image_url
 
 
-    @property
-    def image_urls(self):
+    def prepend_host(self, request, url):
+        host = request.get_host()
+        absolute_url = '{0}://{1}{2}'.format(request.scheme, host, url)
+        return absolute_url
+
+
+    def image_urls(self, request=None):
 
         image_urls = {}
         
         for size_name, image_size in IMAGE_SIZES['all'].items():
             # create the resized image, respecting which side is the longer one
             image_url = self.get_image_url(image_size)
+            if request != None:
+                image_url = self.prepend_host(request, image_url)
             image_urls[size_name] = image_url
 
-        return image_urls
+        return image_urls 
 
 
     def __str__(self):
@@ -540,8 +555,49 @@ class DatasetImages(models.Model):
             return self.dataset.taxon_latname
         
         return 'Dataset Image #{0}'.format(self.id)
-    
+
+
+@receiver(models.signals.post_delete, sender=DatasetImages)
+def auto_delete_image_file_on_delete(sender, instance, **kwargs):
+    '''
+    Deletes file from filesystem
+    when corresponding `DatasetImages` object is deleted.
+    '''
+    if instance.image:
+        resized_images_folder = instance.resized_folder
+        if os.path.isdir(resized_images_folder):
+            shutil.rmtree(resized_images_folder)
+
+        if os.path.isfile(instance.image.path):
+            os.remove(instance.image.path)
+
         
+
+@receiver(models.signals.pre_save, sender=DatasetImages)
+def auto_delete_image_file_on_change(sender, instance, **kwargs):
+    '''
+    Deletes old file from filesystem
+    when corresponding `MediaFile` object is updated
+    with new file.
+    '''
+    if not instance.pk:
+        return False
+
+    try:
+        old_file = DatasetImages.objects.get(pk=instance.pk).image
+    except DatasetImages.DoesNotExist:
+        return False
+
+    new_file = instance.image
+    if not old_file == new_file:
+        if os.path.isfile(old_file.path):
+            os.remove(old_file.path)
+
+        folder_path = os.path.dirname(old_file.path)
+        old_resized_images_folder = os.path.join(folder_path, instance.resized_folder_name)
+        if os.path.isdir(old_resized_images_folder):
+            shutil.rmtree(old_resized_images_folder)
+
 '''
     USER Geometry
     max 3 per user
