@@ -5,7 +5,7 @@ from functools import wraps
 def ajax_required(function):
     @wraps(function)
     def decorator(request, *args, **kwargs):
-        if not request.is_ajax():
+        if not request.headers.get('x-requested-with') == 'XMLHttpRequest':
             return HttpResponseBadRequest('Bad request')
         return function(request, *args, **kwargs)
 
