@@ -24,6 +24,7 @@ urlpatterns = [
     path('password-reset/done/', auth_views.PasswordResetDoneView.as_view(
         extra_context={'base_template': 'base.html'},
         template_name='localcosmos_server/registration/password_reset_done.html'), name='password_reset_done'),
+        
     path('reset/<uidb64>/<token>/', auth_views.PasswordResetConfirmView.as_view(
         extra_context={'base_template': 'base.html'},
         template_name='localcosmos_server/registration/password_reset_confirm.html'), name='password_reset_confirm'),
@@ -31,6 +32,15 @@ urlpatterns = [
         extra_context={'base_template': 'base.html'},
         template_name='localcosmos_server/registration/password_reset_complete.html'),
         name='password_reset_complete'),
+    # password reset with app specific look
+    path('<str:app_uid>/app-password-reset/', views.AppPasswordResetView.as_view(),
+        name='app_password_reset'),
+    path('<str:app_uid>/app-password-reset/<uidb64>/<token>/', views.AppPasswordResetConfirmView.as_view(),
+        name='app_password_reset_confirm'),
+    path('<str:app_uid>/app-password-reset/done/', views.AppPasswordResetDoneView.as_view(),
+        name='app_password_reset_done'),
+    path('<str:app_uid>/app-password-reset/complete/', views.AppPasswordResetCompleteView.as_view(),
+        name='app_password_reset_complete'),
     # server content images
     path('manage-server-content-image/<int:content_type_id>/<int:object_id>/',
         views.ManageServerContentImage.as_view(), name='manage_server_content_image'),
@@ -42,10 +52,8 @@ urlpatterns = [
         views.DeleteServerContentImage.as_view(), name='delete_server_content_image'),
     # SETUP GUI
     path('', include('localcosmos_server.setup_urls')),
-    
     # SERVER CONTROL PANEL
     path('control-panel/', include('localcosmos_server.server_control_panel.urls', namespace='scp')),
-
     # LEGAL
     path('legal-notice/<str:app_uid>/', views.LegalNotice.as_view(), name='legal_notice'),
     path('privacy-statement/<str:app_uid>/', views.PrivacyStatement.as_view(), name='privacy_statement'),
