@@ -2,6 +2,8 @@ from django.conf import settings
 from django.db import models
 from django.contrib.auth.models import AbstractUser, UserManager
 from django.utils.translation import gettext_lazy as _
+from django.db.models import Q
+from django.utils import timezone
 from django.urls import reverse
 from django.db import transaction
 
@@ -13,16 +15,124 @@ from .taxonomy.lazy import LazyAppTaxon
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes.fields import GenericRelation, GenericForeignKey
 
-
-# online ocntent
-from django.template import Template, TemplateDoesNotExist
-from django.template.backends.django import DjangoTemplates
-
 from localcosmos_server.slugifier import create_unique_slug
+from localcosmos_server.utils import generate_md5, get_content_instance_app
 
 from content_licencing.models import ContentLicenceRegistry
 
+from datetime import timedelta
+import requests
+
 import uuid, os, json, shutil
+
+# also used by app_kit
+IMAGE_SIZES = {
+    'regular' : {
+        '1x' : 250,
+        '2x' : 500,
+        #'4x' : 1000,
+    },
+    'large' : {
+        '4x' : 1000,
+    },
+    'xlarge' : {
+        '8x' : 2000,
+    },
+    'all' : {
+        '1x' : 250,
+        '2x' : 500,
+        '4x' : 1000,
+        '8x' : 2000,
+    }
+}
+
+
+
+'''
+    Generic Content Images
+'''
+class ServerContentImageMixin:
+
+    def get_model(self):
+        return ServerContentImage
+
+
+    def get_content_type(self):
+        return ContentType.objects.get_for_model(self.__class__)
+
+    def _content_images(self, image_type='image'):
+
+        content_type = self.get_content_type()
+        ContentImageModel = self.get_model()
+
+        self.content_images = ContentImageModel.objects.filter(content_type=content_type, object_id=self.pk,
+                                                          image_type=image_type).order_by('position')
+
+        return self.content_images
+
+    def all_images(self):
+
+        content_type = self.get_content_type()
+        ContentImageModel = self.get_model()
+
+        self.content_images = ContentImageModel.objects.filter(
+            content_type=content_type, object_id=self.pk)
+
+        return self.content_images
+
+    def images(self, image_type='image'):
+        return self._content_images(image_type=image_type)
+
+    def image(self, image_type='image'):
+        content_image = self._content_images(image_type=image_type).first()
+
+        if content_image:
+            return content_image
+
+        return None
+
+    def image_url(self, size=400):
+
+        content_image = self.image()
+
+        if content_image:
+            return content_image.image_url(size)
+
+        return static('noimage.png')
+
+    # this also deletes ImageStore entries and images on disk
+
+    def delete_images(self):
+
+        content_type = self.get_content_type()
+        ContentImageModel = self.get_model()
+
+        content_images = ContentImageModel.objects.filter(
+            content_type=content_type, object_id=self.pk)
+
+        for image in content_images:
+            # delete model db entries
+            image_store = image.image_store
+            image.delete()
+
+            image_is_used = ContentImageModel.objects.filter(
+                image_store=image_store).exists()
+
+            if not image_is_used:
+                image_store.delete()
+
+    def get_content_images_primary_localization(self):
+
+        locale = {}
+
+        content_images = self.images()
+
+        for content_image in content_images:
+
+            if content_image.text and len(content_image.text) > 0:
+                locale[content_image.text] = content_image.text
+
+        return locale
     
 
 class LocalcosmosUserManager(UserManager):
@@ -51,14 +161,14 @@ class LocalcosmosUserManager(UserManager):
         return superuser
 
 
-class LocalcosmosUser(AbstractUser):
+class LocalcosmosUser(ServerContentImageMixin, AbstractUser):
 
     uuid = models.UUIDField(default=uuid.uuid4, unique=True, editable=False)
     slug = models.SlugField(unique=True)
 
-    details = models.JSONField(null=True)
+    details = models.JSONField(null=True, blank=True)
     
-    follows = models.ManyToManyField('self')
+    follows = models.ManyToManyField('self', blank=True)
 
     is_banned = models.BooleanField(default=False)
 
@@ -68,13 +178,22 @@ class LocalcosmosUser(AbstractUser):
     # there is a bug in django for Dataset.user.on_delete=models.SET_NULL (django 3.1)
     # anonymize the datasets here instead of letting django call SET_NULL
     def anonymize_datasets(self):
-        from localcosmos_server.datasets.models import Dataset
+        from localcosmos_server.datasets.models import Dataset, DatasetImages
         datasets = Dataset.objects.filter(user=self)
         for dataset in datasets:
+
+            # delete all images of this dataset, also remove it from disk
+            # this is due to legal implications
+            images = DatasetImages.objects.filter(dataset=dataset)
+
             dataset.user = None
 
         Dataset.objects.bulk_update(datasets, ['user'])
 
+    def dataset_count(self):
+        from localcosmos_server.datasets.models import Dataset
+        return Dataset.objects.filter(user=self).count()
+
     # do not alter the delete method
     def delete(self, using=None, keep_parents=False):
 
@@ -176,6 +295,9 @@ class App(models.Model):
 
     # url for downloading the currently released apk
     apk_url = models.URLField(null=True)
+    
+    # url for downloading the currently released aab
+    aab_url = models.URLField(null=True)
 
     # url for downloading the currently released ipa
     # as of 2019 this does not make any sense, because apple does not support ad-hoc installations
@@ -221,6 +343,10 @@ class App(models.Model):
     def media_base_path(self):
         return os.path.join(settings.MEDIA_ROOT, self.uid)
 
+    @property
+    def media_base_url(self):
+        return os.path.join(settings.MEDIA_URL, self.uid)
+
 
     def get_url(self):
         if settings.LOCALCOSMOS_PRIVATE == True:
@@ -277,6 +403,9 @@ class App(models.Model):
 
         elif app_state == 'preview':
             root = self.preview_version_path
+            
+            if not root:
+                root = os.path.join(settings.LOCALCOSMOS_APPS_ROOT, self.uid, 'preview', 'www')      
 
         elif app_state == 'review':
             root= self.review_version_path
@@ -300,6 +429,24 @@ class App(models.Model):
             app_settings = json.loads(settings_file.read())
 
         return app_settings
+    
+    
+    def get_licence_registry(self, app_state='preview'):
+        
+        if settings.LOCALCOSMOS_PRIVATE == True:
+            app_state = 'published'
+
+        if app_state == 'published':
+            root = self.get_installed_app_path(app_state)
+            
+            registry_json_path = os.path.join(root, 'localcosmos/licence_registry.json')
+
+            with open(registry_json_path, 'r') as registry_file:
+                app_settings = json.loads(registry_file.read())
+
+            return app_settings
+
+        return {}
 
 
     # read app features from disk, only published apps
@@ -307,7 +454,7 @@ class App(models.Model):
     # used eg by AppTaxonSearch.py
     def get_features(self, app_state='preview'):
 
-        if app_state == 'preview':
+        if settings.LOCALCOSMOS_PRIVATE == False and app_state == 'preview':
             features = {}
 
         else:
@@ -321,6 +468,75 @@ class App(models.Model):
         return features
 
 
+    # privacyPolicy and legalNotice are required
+    def get_frontend(self):
+        app_state = None
+        frontend = None
+
+        if self.published_version_path:
+            app_state = 'published'
+
+        elif self.review_version_path:
+            app_state = 'review'
+        
+        if app_state != None:
+            app_root = self.get_installed_app_path(app_state=app_state)
+
+            features = self.get_features(app_state=app_state)
+
+            frontend_relative_path = features['Frontend']['path']
+            frontend_path = os.path.join(app_root, frontend_relative_path.lstrip('/'))
+
+            with open(frontend_path, 'r') as frontend_file:
+                frontend = json.loads(frontend_file.read())
+        
+        return frontend
+        
+
+    def get_legal_frontend_text(self, key):
+
+        text = ''
+
+        frontend = self.get_frontend()
+        if frontend:
+            text = frontend['userContent']['texts'][key]
+        
+        return text
+
+
+    def get_legal_notice(self):
+        return self.get_legal_frontend_text('legalNotice')
+
+
+    def get_privacy_policy(self):
+        return self.get_legal_frontend_text('privacyPolicy')
+
+
+    @property
+    def logo_url(self):
+        app_state = 'review'
+        features = self.get_features(app_state=app_state)
+        if not features:
+            app_state = 'published'
+            features = self.get_features(app_state=app_state)
+
+        if features:
+            frontend_path = features['Frontend']['path'].lstrip('/')
+            installed_app_path = self.get_installed_app_path(app_state)
+            frontend_filepath = os.path.join(installed_app_path, frontend_path)
+
+            if os.path.isfile(frontend_filepath):
+                
+                with open(frontend_filepath, 'r') as frontend_file:
+                    frontend = json.loads(frontend_file.read())
+                    logo = frontend['userContent']['images'].get('logo', None)
+
+                    if logo:
+                        return logo['imageUrl']['1x'].lstrip('/')
+
+        return None
+
+
     def languages(self):
         languages = [self.primary_language]
         secondary_languages = SecondaryAppLanguages.objects.filter(app=self).values_list('language_code', flat=True)
@@ -342,13 +558,41 @@ class App(models.Model):
                 return locale.get(key, None)
 
         return None
+    
+
+    def get_vernacular(self, taxon, language):
+
+        app_settings = self.get_settings(app_state='published')
+        features = self.get_features(app_state='published')
+
+        vernacular_name = None
+
+        if language not in features['BackboneTaxonomy']['vernacularLookup']:
+            language = app_settings['PRIMARY_LANGUAGE']
+        
+        if language in features['BackboneTaxonomy']['vernacularLookup']:
+            relpath = features['BackboneTaxonomy']['vernacularLookup'][language].lstrip('/')
+
+            vernacular_lookup_path = os.path.join(self.published_version_path, relpath)
+
+            if os.path.isfile(vernacular_lookup_path):
+                with open(vernacular_lookup_path, 'r') as f:
+                    vernacular_lookup = json.loads(f.read())
+                    vernacular_names = vernacular_lookup.get(taxon.name_uuid, None)
+
+                    if vernacular_names:
+                        return vernacular_names['primary']
+                
+        return vernacular_name
+
 
     # LC PRIVATE: remove all contents from disk
     def delete(self, *args, **kwargs):
         app_folder = os.path.join(settings.LOCALCOSMOS_APPS_ROOT, self.uid)
+        super().delete(*args, **kwargs)
+        
         if os.path.isdir(app_folder):
             shutil.rmtree(app_folder)
-        super().delete(*args, **kwargs)
         
 
     def __str__(self):
@@ -398,6 +642,16 @@ class TaxonomicRestrictionBase(ModelWithRequiredTaxon):
     content = GenericForeignKey('content_type', 'object_id')
 
     restriction_type = models.CharField(max_length=100, choices=TAXONOMIC_RESTRICTION_TYPES, default='exists')
+    
+    def serialize(self):
+        return {
+            'taxonSource' : self.taxon_source,
+            'taxonLatname' : self.taxon_latname,
+            'taxonAuthor' : self.taxon_author,
+            'nameUuid' : self.name_uuid,
+            'taxonNuid' : self.taxon_nuid,
+            'restrictionType' : self.restriction_type,
+        }
 
     def __str__(self):
         return self.taxon_latname
@@ -447,108 +701,24 @@ class TaxonomicRestriction(TaxonomicRestrictionBase):
     objects = TaxonomicRestrictionManager()
 
 
-
-
-'''
-    Generic Content Images
-'''
-class ServerContentImageMixin:
-
-    def get_model(self):
-        return ServerContentImage
-
-
-    def get_content_type(self):
-        return ContentType.objects.get_for_model(self.__class__)
-
-    def _content_images(self, image_type='image'):
-
-        content_type = self.get_content_type()
-        ContentImageModel = self.get_model()
-
-        self.content_images = ContentImageModel.objects.filter(content_type=content_type, object_id=self.pk,
-                                                          image_type=image_type).order_by('position')
-
-        return self.content_images
-
-    def all_images(self):
-
-        content_type = self.get_content_type()
-        ContentImageModel = self.get_model()
-
-        self.content_images = ContentImageModel.objects.filter(
-            content_type=content_type, object_id=self.pk)
-
-        return self.content_images
-
-    def images(self, image_type='image'):
-        return self._content_images(image_type=image_type)
-
-    def image(self, image_type='image'):
-        content_image = self._content_images(image_type=image_type).first()
-
-        if content_image:
-            return content_image
-
-        return None
-
-    def image_url(self, size=400):
-
-        content_image = self.image()
-
-        if content_image:
-            return content_image.image_url(size)
-
-        return static('noimage.png')
-
-    # this also deletes ImageStore entries and images on disk
-
-    def delete_images(self):
-
-        content_type = self.get_content_type()
-        ContentImageModel = self.get_model()
-
-        content_images = ContentImageModel.objects.filter(
-            content_type=content_type, object_id=self.pk)
-
-        for image in content_images:
-            # delete model db entries
-            image_store = image.image_store
-            image.delete()
-
-            image_is_used = ContentImageModel.objects.filter(
-                image_store=image_store).exists()
-
-            if not image_is_used:
-                image_store.delete()
-
-    def get_content_images_primary_localization(self):
-
-        locale = {}
-
-        content_images = self.images()
-
-        for content_image in content_images:
-
-            if content_image.text and len(content_image.text) > 0:
-                locale[content_image.text] = content_image.text
-
-        return locale
-
-
-
-
-
 def get_image_store_path(instance, filename):
     blankname, ext = os.path.splitext(filename)
 
-    new_filename = '{0}{1}'.format(instance.md5, ext)
+    md5 = instance.md5
+
+    if not md5:
+        md5 = generate_md5(instance.source_image)
+        #hashlib.md5(instance.source_image.read()).hexdigest()
+        # this line is extremely required. do not delete it. otherwise the file will not be read correctly
+        # this now done by generate_md5
+        #instance.source_image.seek(0)
+
+    new_filename = '{0}{1}'.format(md5, ext)
     path = '/'.join(['localcosmos-server', 'imagestore', '{0}'.format(instance.uploaded_by.pk),
                      new_filename])
     return path
 
 
-
 class ImageStoreAbstract(ModelWithTaxon):
 
     LazyTaxonClass = LazyAppTaxon
@@ -572,7 +742,6 @@ class ServerImageStore(ImageStoreAbstract):
     source_image = models.ImageField(upload_to=get_image_store_path)
 
 
-
 class ContentImageAbstract(models.Model):
 
     crop_parameters = models.TextField(null=True)
@@ -592,12 +761,30 @@ class ContentImageAbstract(models.Model):
     position = models.IntegerField(default=0)
     is_primary = models.BooleanField(default=False)
 
-    # only primary language
+    # caption, text below image
     text = models.CharField(max_length=355, null=True)
 
     # flag if a translation is needed
     requires_translation = models.BooleanField(
         default=False)  # not all images require a translation
+    
+    # SEO
+    title = models.CharField(max_length=355, null=True, blank=True)
+    alt_text = models.TextField(null=True, blank=True)
+    
+    
+    def save(self, *args, **kwargs):
+        
+        if self.is_primary == True:
+            
+            old_primaries = self.__class__.objects.filter(content_type=self.content_type, object_id=self.object_id, image_type=self.image_type, is_primary=True)
+            
+            if self.pk:
+                old_primaries.exclude(pk=self.pk)
+                
+            old_primaries.update(is_primary=False)
+        
+        super().save(*args, **kwargs)
 
 
     class Meta:
@@ -710,7 +897,7 @@ class ContentImageProcessing:
         else:
             cropped_canvas = canvas_with_features
 
-        cropped_canvas.thumbnail([width, height], Image.ANTIALIAS)
+        cropped_canvas.thumbnail([width, height], Image.LANCZOS)
 
         if original_image.format != 'PNG':
             cropped_canvas = cropped_canvas.convert('RGB')
@@ -749,9 +936,277 @@ class ContentImageProcessing:
                 self.image_store.source_image.url), 'thumbnails', thumbname)
 
         return thumburl
+    
+    
+    def image_urls(self, image_sizes=['regular', 'large']):
+        
+        image_urls = {}
+        
+        for image_sizes_key in image_sizes:
+            for size_name, size in IMAGE_SIZES[image_sizes_key].items():
+                
+                image_url = self.image_url(size)
+                
+                image_urls[size_name] = image_url
+        
+        return image_urls
+
+
+    def srcset(self, request=None, force=False):
+        
+        srcset = {
+            '1x' : self.image_url(size=200, force=force),
+            '2x' : self.image_url(size=400, force=force),
+        }
+
+        if request:
+
+            host = request.get_host()
+
+            for key, url in srcset.items():
+                absolute_url = '{0}://{1}{2}'.format(request.scheme, host, url)
+                srcset[key] = absolute_url
+        
+
+        return srcset
 
 
 
 class ServerContentImage(ContentImageProcessing, ContentImageAbstract):
 
-    image_store = models.ForeignKey(ServerImageStore, on_delete=models.CASCADE)
\ No newline at end of file
+    image_store = models.ForeignKey(ServerImageStore, on_delete=models.CASCADE)
+    
+
+'''--------------------------------------------------------------------------------------------------------------
+    SEO
+    - seo parameters for pages (not images)
+    - as it should be reusable for multiple models without modifying the database schema of those models
+      GenericRelation is used instead of ForeignKey
+--------------------------------------------------------------------------------------------------------------'''
+class SeoParametersAbstract(models.Model):
+
+    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
+    object_id = models.PositiveIntegerField()
+    content_object = GenericForeignKey('content_type', 'object_id')
+    
+    title = models.CharField(max_length=355, null=True, blank=True)
+    meta_description = models.TextField(null=True, blank=True)
+    
+    def __str__(self):
+        return f"{self.title or 'SEO Parameters'} for {self.content_object}"
+    
+    class Meta:
+        unique_together = ('content_type', 'object_id')
+        verbose_name = "SEO Parameter"
+        verbose_name_plural = "SEO Parameters"
+        abstract = True
+
+
+class ServerSeoParameters(SeoParametersAbstract):
+    pass
+
+
+'''--------------------------------------------------------------------------------------------------------------
+    EXTERNAL MEDIA
+--------------------------------------------------------------------------------------------------------------'''
+EXTERNAL_MEDIA_TYPES = (
+    ('image', _('Image')),
+    ('youtube', _('Youtube')),
+    ('vimeo', _('Vimeo')),
+    ('mp3', _('MP3 File')),
+    ('wav', _('WAV File')),
+    ('pdf', _('PDF Document')),
+    ('website', _('Website')),
+    ('file', _('File')), # generic file, download links
+)
+
+EXTERNAL_MEDIA_CATEGORIES = (
+    ('video', _('Video')),
+    ('audio', _('Audio')),
+    ('document', _('Document')),
+)
+
+# Mapping from media types to categories
+# Only map types that have meaningful categories (multiple subtypes)
+MEDIA_TYPE_TO_CATEGORY = {
+    'youtube': 'video',
+    'vimeo': 'video',
+    'mp3': 'audio',
+    'wav': 'audio',
+    'pdf': 'document',
+    # image and website don't map to categories - they stand alone
+}
+
+class ExternalMediaManager(models.Manager):
+    
+    def needs_checking(self, days=30):
+        cutoff = timezone.now() - timedelta(days=days)
+        return self.filter(
+            Q(last_checked_at__lt=cutoff) | Q(last_checked_at__isnull=True)
+        )
+        
+class ExternalMediaAbstract(models.Model):
+    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
+    object_id = models.PositiveIntegerField()
+    content_object = GenericForeignKey('content_type', 'object_id')
+    url = models.URLField(max_length=2000)
+    title = models.CharField(max_length=255)
+    caption = models.CharField(max_length=255, null=True, blank=True)
+    alt_text = models.CharField(max_length=355, null=True, blank=True)
+    media_type = models.CharField(max_length=50, choices=EXTERNAL_MEDIA_TYPES)
+    media_category = models.CharField(max_length=50, choices=EXTERNAL_MEDIA_CATEGORIES, null=True, blank=True)
+    author = models.CharField(max_length=255, null=True, blank=True)
+    licence = models.TextField(null=True, blank=True)
+    position = models.IntegerField(default=0)
+    file_size = models.BigIntegerField(null=True, blank=True, help_text=_('File size in bytes'))
+    last_checked_at = models.DateTimeField(null=True, blank=True)
+    is_accessible = models.BooleanField(default=True)
+
+    objects = ExternalMediaManager()
+    
+    def __str__(self):
+        return f"{self.title or self.url} ({self.media_type})"
+    
+    def save(self, *args, **kwargs):
+        # Auto-fill media_category based on media_type if not already set
+        # Only set category if the media_type has meaningful categorization
+        if not self.media_category and self.media_type:
+            self.media_category = MEDIA_TYPE_TO_CATEGORY.get(self.media_type)
+        
+        # Check if this is a new object or if URL has changed
+        url_changed = False
+        if self.pk is None:  # New object
+            url_changed = True
+        else:
+            # Check if URL changed
+            try:
+                old_instance = self.__class__.objects.get(pk=self.pk)
+                url_changed = old_instance.url != self.url
+            except self.__class__.DoesNotExist:
+                url_changed = True
+        
+        # If URL changed and we don't have file size, try to fetch it
+        if url_changed and not self.file_size and self.url:
+            self.update_file_size(save=False)
+            
+        super().save(*args, **kwargs)
+    
+    def get_media_category_display_name(self):
+        """Get the human-readable category name"""
+        if self.media_category:
+            category_dict = dict(EXTERNAL_MEDIA_CATEGORIES)
+            return category_dict.get(self.media_category, self.media_category)
+        else:
+            # For standalone types, return the media type display name
+            media_type_dict = dict(EXTERNAL_MEDIA_TYPES)
+            return media_type_dict.get(self.media_type, self.media_type)
+    
+    def get_file_size_display(self):
+        """Return human-readable file size"""
+        if not self.file_size:
+            return _('Unknown size')
+        
+        # Convert bytes to human readable format
+        size = self.file_size
+        for unit in ['B', 'KB', 'MB', 'GB']:
+            if size < 1024.0:
+                return f"{size:.1f} {unit}"
+            size /= 1024.0
+        return f"{size:.1f} TB"
+    
+    def is_large_file(self, threshold_mb=5):
+        """Check if file is considered large (default: 5MB)"""
+        if not self.file_size:
+            return False
+        threshold_bytes = threshold_mb * 1024 * 1024
+        return self.file_size > threshold_bytes
+    
+    def fetch_file_size(self):
+        """Fetch file size from URL without downloading the entire file"""
+        
+        if not self.url:
+            return None
+            
+        try:
+            # For YouTube and other platform URLs, size determination is complex
+            if self.media_type == 'youtube':
+                # YouTube file sizes vary by quality, can't determine easily
+                return None
+            
+            # For direct file URLs, use HEAD request to get Content-Length
+            response = requests.head(self.url, timeout=10, allow_redirects=True)
+            
+            if response.status_code == 200:
+                content_length = response.headers.get('Content-Length')
+                if content_length:
+                    return int(content_length)
+            
+            # If HEAD doesn't work, try GET with range header
+            headers = {'Range': 'bytes=0-0'}
+            response = requests.get(self.url, headers=headers, timeout=10, stream=True)
+            
+            if response.status_code == 206:  # Partial Content
+                content_range = response.headers.get('Content-Range')
+                if content_range:
+                    # Content-Range: bytes 0-0/12345 (total size is 12345)
+                    total_size = content_range.split('/')[-1]
+                    if total_size.isdigit():
+                        return int(total_size)
+                        
+        except Exception as e:
+            # Log the error in production
+            # logger.warning(f"Could not fetch file size for {self.url}: {e}")
+            pass
+            
+        return None
+    
+    def update_file_size(self, save=True):
+        """Update the file_size field by fetching it from the URL"""
+        size = self.fetch_file_size()
+        if size is not None:
+            self.file_size = size
+            if save:
+                self.save(update_fields=['file_size'])
+            return True
+        return False
+    
+    def check_url_and_update_metadata(self):
+        """Check URL accessibility and update metadata including file size"""
+
+        
+        if not self.url:
+            return False
+            
+        try:
+            # Update file size
+            size_updated = self.update_file_size(save=False)
+            
+            # Check accessibility
+            response = requests.head(self.url, timeout=10, allow_redirects=True)
+            self.is_accessible = response.status_code == 200
+            self.last_checked_at = timezone.now()
+            
+            # Save all changes at once
+            update_fields = ['is_accessible', 'last_checked_at']
+            if size_updated:
+                update_fields.append('file_size')
+                
+            self.save(update_fields=update_fields)
+            return True
+            
+        except Exception as e:
+            self.is_accessible = False
+            self.last_checked_at = timezone.now()
+            self.save(update_fields=['is_accessible', 'last_checked_at'])
+            return False
+
+    class Meta:
+        verbose_name = _('External Media')
+        verbose_name_plural = _('External Media')
+        ordering = ['position', 'id']
+        unique_together = ('content_type', 'object_id', 'url')
+        abstract = True
+
+
+class ServerExternalMedia(ExternalMediaAbstract):
+    pass
\ No newline at end of file
